/******************************************************************************
*
* project name:  TICT Game Construction Kit (TICTGCK)
* file name:     init.c
* initial date:  27/06/2001
* author:        thomas.nussbaumer@gmx.net
*
* $Id$
*
******************************************************************************/

#include <intr.h>
#include <kbd.h>
#include <compat.h>
#include <alloc.h>
#include <gray.h>
#include <string.h>

#include "gcklib.h"

//-----------------------------------------------------------------------------
// The following variable is used to embed the library version as string into
// every executable wich linked against the GCKLIB library and calls at least
// InitGCKLibrary().
//-----------------------------------------------------------------------------
volatile unsigned char ____vnr____[] = GCKLIB_VERSION;


unsigned char* __backbuffer1__    = NULL;
unsigned char* __backbuffer2__    = NULL;
short          __displayoffset__  = 0;

//-----------------------------------------------------------------------------
// The keyboard reading routine may have a negative effect on timers, because
// the INT5 handler is redirected to a dummy handler during its request
//-----------------------------------------------------------------------------
KBSTATE (*__readkeys__)(void)   = NULL;   // will hold key function for calctype
void    (*__sleep__ms__)(unsigned short) = NULL;

//-----------------------------------------------------------------------------
// if INT5 is called approx. 20 times per second, variable __time__base__ will
// need 59652 hours to overflow. Therefore there is no need for any overflow
// checking!
//-----------------------------------------------------------------------------
volatile unsigned long __time__base__ = 0L;

static INT_HANDLER __old__int1__   = NULL;   // holds old INT1 handler
static INT_HANDLER __old__int5__   = NULL;   // holds old INT5 handler
short              __int1__state__ = DEFAULT_INT1_EXEC_STATE;
short              __int5__state__ = DEFAULT_INT5_EXEC_STATE;


/*===========================================================================*/
/* "sleeps" for a specific number of ticks (1 tick == approx. 50 ms)         */
/*===========================================================================*/
void SleepForTicks(unsigned long nr_ticks) {
    register unsigned long wait_until = GetActualTicks() + nr_ticks;
    while (wait_until < GetActualTicks());
}


/*===========================================================================*/
/* get hardware version of calculator                                        */
/*===========================================================================*/
unsigned long GetHardwareVersion(void) {
    unsigned long  *rombase = (unsigned long *)((*(unsigned long *)0xC8) & 0x600000);
    unsigned long   hwpb = rombase[65];
    return hwpb - (unsigned long)rombase  < 0x10000 && *(unsigned short*)hwpb > 22 ? *(unsigned long*)(hwpb + 22) : 1;
}


/*===========================================================================*/
/* "sleeps" for a specific number of milliseconds on HW1 calcs               */
/*                                                                           */
/* Calibrated to need exactly 10 seconds for input value 10000. Smaller      */
/* input values may differ a little bit due to the overhead of the function  */
/* call.                                                                     */
/*===========================================================================*/
void SleepMS_HW1(unsigned short);
asm("xdef SleepMS_HW1\n"
"SleepMS_HW1: movem.l %d1-%d3,-(%sp)\n"
"             move.w  (16,%sp),%d2\n"
"             move.l  #31,%d1\n"
"             move.l  #31,%d3\n"
"__wl2__:     move.w  #380,%d0       /* value evaluated by tests */\n"
"__wl1__:     rol.l   %d3,%d1\n"
"             dbra    %d0,__wl1__\n"
"             dbra    %d2,__wl2__\n"
"             movem.l (%sp)+,%d1-%d3\n"
"             rts");


/*===========================================================================*/
/* "sleeps" for a specific number of milliseconds on HW2 calcs               */
/*                                                                           */
/* Calibrated to need exactly 10 seconds for input value 10000. Smaller      */
/* input values may differ a little bit due to the overhead of the function  */
/* call.                                                                     */
/*===========================================================================*/
void SleepMS_HW2(unsigned short);
asm("xdef SleepMS_HW2\n"
"SleepMS_HW2: movem.l %d1-%d3,-(%sp)\n"
"             move.w  (16,%sp),%d2\n"
"             move.l  #31,%d1\n"
"             move.l  #31,%d3\n"
"__w2l2__:    move.w  #456,%d0         /* 456 = 380 * 12/10 */\n"
"__w2l1__:    rol.l   %d3,%d1\n"
"             dbra    %d0,__w2l1__\n"
"             dbra    %d2,__w2l2__\n"
"             movem.l (%sp)+,%d1-%d3\n"
"             rts");


/*===========================================================================*/
/* own interrupt handler for INT1 which calls old handler if wanted          */
/*===========================================================================*/
DEFINE_INT_HANDLER (__InternalInt1Handler__) {
    if (__int1__state__) ExecuteHandler(__old__int1__);
}


/*===========================================================================*/
/* own interrupt handler for INT5 which calls old handler if wanted          */
/*===========================================================================*/
DEFINE_INT_HANDLER (__InternalInt5Handler__) {
    __time__base__++;
    if (__int5__state__) ExecuteHandler(__old__int5__);
}


/*===========================================================================*/
/* returns KBSTATE with key flags set or 0 (TI-89 VERSION)                   */
/*===========================================================================*/
KBSTATE GetPossibleKeys89(void) {
    register unsigned char val;
    register short         oldint1state = GetDefaultINT1Exec();
    register short         oldint5state = GetDefaultINT5Exec();
    register KBSTATE       key = 0;

    // disable calling of default INT1/INT5 handlers temporary
    SetDefaultINT1Exec(0);
    SetDefaultINT5Exec(0);

    val = _rowread(~(1<<0));
    if (val & 0x01) key |= KEYFLAGS_UP;     // up
    if (val & 0x02) key |= KEYFLAGS_LEFT;   // down
    if (val & 0x04) key |= KEYFLAGS_DOWN;   // left
    if (val & 0x08) key |= KEYFLAGS_RIGHT;  // right
    if (val & 0x10) key |= KEYFLAGS_KA;     // 2nd
    if (val & 0x40) key |= KEYFLAGS_KB;     // Diamond
    if (val & 0x20) key |= KEYFLAGS_KE;     // Shift
    if (val & 0x80) key |= KEYFLAGS_KF;     // Alpha
    val = _rowread(~(1<<1));
    if (val & 0x02) key |= KEYFLAGS_PLUS;   // +
    if (val & 0x04) key |= KEYFLAGS_MINUS;  // -
    val =_rowread(~(1<<4));
    if (val & 0x40) key |= KEYFLAGS_KG;     // Mode
    if (val & 0x20) key |= KEYFLAGS_KH;     // Y
    val = _rowread(~(1<<5));
    if (val & 0x01) key |= KEYFLAGS_MODE;   // APPS
    if (val & 0x40) key |= KEYFLAGS_KC;     // Home
    if (val & 0x20) key |= KEYFLAGS_KD;     // X
    if (_rowread(~(1<<6)) & 0x01) key |= KEYFLAGS_ESC;    // ESC

    // restore previous values of default INT1/INT5 handlers execution
    SetDefaultINT1Exec(oldint1state);
    SetDefaultINT5Exec(oldint5state);

    return key;
}


/*===========================================================================*/
/* returns KBSTATE with key flags set or 0 (TI-92p VERSION)                  */
/*===========================================================================*/
KBSTATE GetPossibleKeys92p(void) {
    register unsigned char val;
    register short         oldint1state = GetDefaultINT1Exec();
    register short         oldint5state = GetDefaultINT5Exec();
    register KBSTATE       key = 0;

    // disable calling of default INT1/INT5 handlers temporary
    SetDefaultINT1Exec(0);
    SetDefaultINT5Exec(0);

    val = _rowread(~(1<<0));
    if (val & 0x10) key |= KEYFLAGS_LEFT;             //up
    if (val & 0x20) key |= KEYFLAGS_UP;               //down
    if (val & 0x40) key |= KEYFLAGS_RIGHT;            //left
    if (val & 0x80) key |= KEYFLAGS_DOWN;             //right
    if (_rowread(~(1<<1)) & 0x10) key |= KEYFLAGS_KH; //F8
    if (_rowread(~(1<<2)) & 0x10) key |= KEYFLAGS_KC; //F3
    if (_rowread(~(1<<3)) & 0x10) key |= KEYFLAGS_KG; //F7
    if (_rowread(~(1<<4)) & 0x10) key |= KEYFLAGS_KB; //F2
    if (_rowread(~(1<<5)) & 0x10) key |= KEYFLAGS_KF; //F6
    if (_rowread(~(1<<6)) & 0x10) key |= KEYFLAGS_KA; //F1
    val = _rowread(~(1<<7));
    if (val & 0x40) key |= KEYFLAGS_MODE;             //APPS
    if (val & 0x10) key |= KEYFLAGS_KE;               //F5
    val = _rowread(~(1<<8));
    if (val & 0x40) key |= KEYFLAGS_ESC;              //ESC
    if (val & 0x10) key |= KEYFLAGS_PLUS;             //+
    val = _rowread(~(1<<9));
    if (val & 0x01) key |= KEYFLAGS_MINUS;            //-
    if (val & 0x10) key |= KEYFLAGS_KD;               //F4

    // restore previous values of default INT1/INT5 handlers execution
    SetDefaultINT1Exec(oldint1state);
    SetDefaultINT5Exec(oldint5state);

    return key;
}

/*===========================================================================*/
/* initializes GCK library                                                   */
/*                                                                           */
/* (1) initialize backbuffer (allocates it) - returns immediately 0 if failed*/
/* (2) installs special INT1 and INT5 handlers to provide calculator type    */
/*     independent lowlevel keyboard reading                                 */
/* (3) initializes timecounter support                                       */
/* (4) initializes HW version independent SleepForMilliseconds routine       */
/* (5) Turns grayscales on                                                   */
/*                                                                           */
/* NOTE: if your program wants to install its own interrupt handlers for     */
/*       INT1 and/or INT5 you have to install them BEFORE you call           */
/*       calling InitGCKLibrary()                                            */
/*===========================================================================*/
short InitGCKLibrary(void) {
    if (!(__backbuffer1__=(unsigned char*)malloc(3840*2))) return 0;
    memset(__backbuffer1__,0,3840*2);
    __backbuffer2__ = __backbuffer1__ + 3840;

    SetActualTicks(0L);  // reset variable __time__base__ (our counter of INT5 calls)

    __int1__state__ = DEFAULT_INT1_EXEC_STATE;
    __int5__state__ = DEFAULT_INT5_EXEC_STATE;
    __old__int1__   = GetIntVec(AUTO_INT_1);
    __old__int5__   = GetIntVec(AUTO_INT_5);
    SetIntVec(AUTO_INT_1,__InternalInt1Handler__);
    SetIntVec(AUTO_INT_5,__InternalInt5Handler__);

    //-------------------------------------------------------------------------
    // turn grayscales on
    //-------------------------------------------------------------------------
    if (!GrayOn()) {
        CleanupGCKLibrary();
        return 0;
    }


    if (TI89) {
        __readkeys__      = GetPossibleKeys89;
        __displayoffset__ = 0;
    }
    else {
        __readkeys__      = GetPossibleKeys92p;
        __displayoffset__ = 4+14*30;
    }

    if (GetHardwareVersion()>1L) __sleep__ms__ = SleepMS_HW2;
    else                         __sleep__ms__ = SleepMS_HW1;

    return 1;
}


/*===========================================================================*/
/* cleanup of GCK library                                                    */
/*                                                                           */
/* (1) turns grayscales off                                                  */
/* (2) frees backbuffer                                                      */
/* (3) restores default INT1 and INT5 handlers                               */
/*                                                                           */
/* NOTE: if your program uses own interrupt handlers for INT1 and/or INT5    */
/*       you have to remove them AFTER calling CleanupGCKLibrary()           */
/*===========================================================================*/
void CleanupGCKLibrary(void) {
    GrayOff();

    if (__backbuffer1__) {
        free(__backbuffer1__);
        __backbuffer1__=NULL;
    }

    SetIntVec(AUTO_INT_1,__old__int1__);
    SetIntVec(AUTO_INT_5,__old__int5__);
}

//=============================================================================
// copies backbuffer to visible screen
//=============================================================================
void CopyBackbuffer2Screen(void) {
    register unsigned char* tmpbuffer1 = __backbuffer1__;
    register unsigned char* tmpbuffer2 = __backbuffer2__;
    register unsigned char* dest1 = GetPlane(0) + __displayoffset__;
    register unsigned char* dest2 = GetPlane(1) + __displayoffset__;

    asm volatile(
        "movem.l %%d0-%%d5,-(%%sp);"\
        "moveq #99,%%d0;"\
        "0:;"\
        "movem.l (%0)+,%%d1-%%d5;"\
        "movem.l %%d1-%%d5,(%1);"\
        "movem.l (%2)+,%%d1-%%d5;"\
        "movem.l %%d1-%%d5,(%3);"\
        "lea (%1,30),%1;"\
        "adda.w #10,%0;"\
        "lea (%3,30),%3;"\
        "adda.w #10,%2;"\
        "dbra %%d0,0b;"\
        "movem.l  (%%sp)+,%%d0-%%d5"
        : "=a" (tmpbuffer1),"=a" (dest1),"=a" (tmpbuffer2),"=a" (dest2)
        : "0" (tmpbuffer1),"1" (dest1),"2" (tmpbuffer2),"3" (dest2));
}


//#############################################################################
// Revision History
//#############################################################################
//
// $Log$
