/******************************************************************************
*
* project name:  TICT Game Construction Kit (TICT-GCK)
* file name:     loadmap.c
* initial date:  27/06/2001
* author:        thomas.nussbaumer@gmx.net
*
* $Id$
*
******************************************************************************/

#include <alloc.h>
#include <string.h>

#include "gcklib.h"


//=============================================================================
// tries to load a map file
//
// the map file has the following format:
//
// unsigned long  MAGIC_MAPFILE (0x4D615066 == MaPf)
// unsigned short size_x
// unsigned short size_y
// unsigned short data[size_x*sizey]
//
// the data is arranged from left to right and up to down
//
// line0/tile0
// line0/tile1
// line0/tile2
// line0/tile3
// ....
//
// (to generate a mapfile the oncalc mapeditor can be used)
//
// NOTE: Don't forget to free() the returned buffer !!!
//=============================================================================
MAPDATA* LoadMapData(char* filename) {
    SYM_ENTRY*       symentry = GetSymByName(filename);
    HANDLE           h;
    unsigned char*   src;
    MAPDATA*         mfh;
    unsigned short   length;

    if (!symentry) return NULL;

    if (!(h = symentry->handle)) return NULL;
    if (!(src = HLock(h)))       return NULL;
    if (!(IS_MAPFILE(src)))      return NULL;

    length = *((unsigned short*)(src+6)) * *((unsigned short*)(src+8)) * 2 + 14;

    if (!(mfh = (MAPDATA*)malloc(length))) {
        HeapUnlock(h);
        return NULL;
    }

    memcpy(mfh,src+6,length);
    HeapUnlock(h);
    return mfh;
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log$
//