/******************************************************************************
*
* project name:  TICT Game Construction Kit (TICT-GCK)
* file name:     loadtiles.c
* initial date:  27/06/2001
* author:        thomas.nussbaumer@gmx.net
*
* $Id$
*
******************************************************************************/

#include <alloc.h>
#include <string.h>
#include "gcklib.h"

//=============================================================================
// tries to load a tiles file
//
// the tiles file has the following format:
//
// unsigned long  MAGIC_TILEFILE (0x54694C65 == TiLe)
// unsigned short nr_tiles
// unsigned short data[nr_tiles*32]
//
// the data is arranged in interleaved format:
//
// tile 1 - lightplane data (16 shorts)
// tile 1 - darkplane  data (16 shorts)
// tile 2 - lightplane data (16 shorts)
// tile 2 - darkplane  data (16 shorts)
// tile 3 - lightplane data (16 shorts)
// tile 3 - darkplane  data (16 shorts)
// ....
//
// (to generate a tile file program tools/tilegen can be used)
//
// NOTE: Don't forget to free() the returned buffer !!!
//=============================================================================
TILEDATA* LoadTileData(char* filename) {
    SYM_ENTRY*       symentry = GetSymByName(filename);
    HANDLE           h;
    unsigned char*   src;
    TILEDATA*        tfh;
    unsigned short   length;

    if (!symentry) return NULL;

    if (!(h = symentry->handle)) return NULL;
    if (!(src = HLock(h)))       return NULL;
    if (!(IS_TILEFILE(src)))     return NULL;

    length = *((unsigned short*)(src+6)) * 64 + 2;

    if (!(tfh = (TILEDATA*)malloc(length))) {
        HeapUnlock(h);
        return NULL;
    }

    memcpy(tfh,src+6,length);
    HeapUnlock(h);
    return tfh;
}


//#############################################################################
// Revision History
//#############################################################################
//
// $Log$
//