/******************************************************************************
*
* project name:  TICT Game Construction Kit (TICT-GCK)
* file name:     puttile16x16clipped.c
* initial date:  27/06/2001
* author:        thomas.nussbaumer@gmx.net
*
* $Id$
*
******************************************************************************/

#include "gcklib.h"

#define MAX_HEIGHT 127
#define MAX_WIDTH  239

//=============================================================================
// tile drawing function for 16x16 pixel sized tiles WITH CLIPPING
//
// sprites are clipped in the range of (0<=x<=239) and (0<=y<=127)
//
// NOTE: for performance reasons the sprite data looks like this:
//
// data for LIGHT_PLANE (16 unsigned short's)
// data for DARK_PLANE  (16 unsigned short's)
//
// !! only 1 pointer is used to hand over the parameter !!
//
// NOTE2: This function operates on a LCD_SIZE*2 large backbuffer which is
//        given by the external variable backbuffer
//=============================================================================
void PutTile16x16Clipped(short x,short y,unsigned short* tiledata) {
    register long            addr1   = (long)__backbuffer1__;
    register long            addr2   = addr1+3840;
    register unsigned short* sprite1 = tiledata;
    register unsigned short* sprite2 = sprite1+16;
    register short           h       = 16;
    register unsigned long   mask; //   = ~(0x0000ffffL<<cnt);
    register long            offset;
    register unsigned short  cnt;

    //-------------------------------------------------------------------------
    // test for completely out of range sprite ...
    //-------------------------------------------------------------------------
    if (x<-15 || x>MAX_WIDTH || y<-15 || y>MAX_HEIGHT) return;

    //-------------------------------------------------------------------------
    // clip above ...
    //-------------------------------------------------------------------------
    if (y<0) {
        sprite1 -= y;
        sprite2 -= y;
        h       += y;
        y        = 0;
    }

    //-------------------------------------------------------------------------
    // clip below ...
    //-------------------------------------------------------------------------
    else if (y+h>MAX_HEIGHT) h = MAX_HEIGHT-y;

    //-------------------------------------------------------------------------
    // clip left ...
    //-------------------------------------------------------------------------
    if (x<0) {
        cnt    =  -x+16;
        offset =  (y<<5)-(y<<1);
        addr1  += offset;
        addr2  += offset;
        mask   =  ~(0x0000ffffL<<cnt);
        for (;h;h--,addr1+=30,addr2+=30) {
            *(long*)addr1&=mask;
            *(long*)addr1|=(long)(*sprite1++)<<cnt;
            *(long*)addr2&=mask;
            *(long*)addr2|=(long)(*sprite2++)<<cnt;
        }
    }
    //-------------------------------------------------------------------------
    // clip right ...
    //-------------------------------------------------------------------------
    else if (x+16>MAX_WIDTH) {
        register unsigned short mask2=0xffff<<(16-(MAX_WIDTH-x));
        cnt    = 16-(x&15);
        mask = ~((0x0000ffffL & mask2)<<cnt);
        offset = (y<<5)-(y<<1)+((x>>3)&0xfffe);
        addr1  += offset;
        addr2  += offset;
        for (;h;h--,addr1+=30,addr2+=30) {
            *(long*)addr1&=mask;
            *(long*)addr1|=(long)(*sprite1++ & mask2)<<cnt;
            *(long*)addr2&=mask;
            *(long*)addr2|=(long)(*sprite2++ & mask2)<<cnt;
        }
    }
    else {
        cnt    = 16-(x&15);
        mask   =  ~(0x0000ffffL<<cnt);
        offset = (y<<5)-(y<<1)+((x>>3)&0xfffe);
        addr1  += offset;
        addr2  += offset;
        for (;h;h--,addr1+=30,addr2+=30) {
            *(long*)addr1&=mask;
            *(long*)addr1|=(long)(*sprite1++)<<cnt;
            *(long*)addr2&=mask;
            *(long*)addr2|=(long)(*sprite2++)<<cnt;
        }
    }
}


//#############################################################################
// Revision History
//#############################################################################
//
// $Log$
//
