/******************************************************************************
*
* project name:  TICT Game Construction Kit (TICT-GCK)
* file name:     editmap.c
* initial date:  29/06/2001
* author:        thomas.nussbaumer@gmx.net
* description:   simple map editor
*
* $Id$
*
******************************************************************************/

//-----------------------------------------------------------------------------
// TODO
//
// (1) modification menu with the following points:
//
//     (a) Select Tilefile
//     (b) Save Map
//     (c) Modify size
//     (d) Show/Hide solid state of tiles
//     (e) Show Help Page
//
// (2) modify the tile selection at the right so that the active tile is in
//     the center of the screen
//
// (3) support for setting the state of a tile to solid (mapentry |= FLAG_SOLID)
//
//-----------------------------------------------------------------------------

#define NO_EXIT_SUPPORT
#define OPTIMIZE_ROM_CALLS

#include <tigcclib.h>
#include "../../lib/gcklib.h"

short _ti92plus;
short _ti89;

short edit_pos_x           = 0;
short edit_pos_y           = 0;
unsigned short active_tile = 0;
unsigned short nr_tiles    = 0;

//-----------------------------------------------------------------------------
// cursor sprite
//-----------------------------------------------------------------------------
unsigned short cursor[16*3] = {
    // plane 0 data
    0xffff,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,
    0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0xffff,
    // plane 1 data
    0xffff,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,
    0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0xffff,
    // mask (NOTE: this is not a normal sprite mask, but an inverted one to gain speed !!)
    0xffff,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,
    0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0x8001,0xffff
};

unsigned short empty[16*3] = {
    // plane 0 data
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    // plane 1 data
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
    // mask (NOTE: this is not a normal sprite mask, but an inverted one to gain speed !!)
    0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,
    0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff
};

//-----------------------------------------------------------------------------
// separator sprite
//-----------------------------------------------------------------------------
unsigned short separator[16*3] = {
    // plane 0 data
    0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,
    0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,
    // plane 1 data
    0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,
    0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,
    // mask (NOTE: this is not a normal sprite mask, but an inverted one to gain speed !!)
    0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,
    0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,0x0003,
};


//=============================================================================
//
//=============================================================================
short MainMenu(void) {
    short handle  = PopupNew("Select Mode",C89_92(53,69));
    short select;

    PopupAddText(handle,-1,"Modify Map",1);
    PopupAddText(handle,-1,"Create Map",2);
    select = PopupDo(handle,CENTER,CENTER,0);
    HeapFree(handle);
    return select;
}


//=============================================================================
//
//=============================================================================
short SizeSelection(void) {
    short handle  = PopupNew("Select Mapsize",C89_92(53,69));
    short select;
    char  buffer[30];

    for (select=10;select<=20;select++) {
        sprintf(buffer,"%d * %d",select,select);
        PopupAddText(handle,-1,buffer,select);
    }
    select = PopupDo(handle,CENTER,CENTER,0);
    HeapFree(handle);
    return select;
}


//=============================================================================
// draws tile selection on the left, the coordinates display in the lower right
// corner and the cursor itself
//=============================================================================
void DrawRestOfScreen(unsigned short* tiles) {
    short i;
    short tilenr = active_tile;

    for (i=0;i<112/16;i++) {
        PutTile16x16Clipped(144,i*16,&tiles[tilenr*32]);
        PutSprite16x16Clipped(128,i*16,separator);
        if (i==0) PutSprite16x16Clipped(144,0,cursor);
        tilenr++;
        if (tilenr>=(short)nr_tiles) tilenr=0;
    }

    PutTile16x16Clipped(144,5*16,empty);
    PutTile16x16Clipped(144,6*16,empty);

    FontSetSys(F_4x6);
    for (i=0;i<2;i++) {
        PortSet(GetBackbuffer(i),239,127);
        printf_xy(144+2,5*16+1,"x/y",ScreenX2MapX(edit_pos_x));
        printf_xy(144+2,5*16+8, "%03d",ScreenX2MapX(edit_pos_x));
        printf_xy(144+2,5*16+14,"%03d",ScreenY2MapY(edit_pos_y));
    }
    PortRestore();

    PutSprite16x16Clipped(edit_pos_x,edit_pos_y,cursor);
}


#define COMMAND_EXIT    0
#define COMMAND_REFRESH 1

//=============================================================================
// handles input
//=============================================================================
short HandleKeys(KBSTATE input) {
    if (input & KEYFLAGS_ESC) {
        WaitForAllKeysReleased();
        return COMMAND_EXIT;
    }

    if (input & KEYFLAGS_UP) {
        if (edit_pos_y>15)  edit_pos_y-=16;
        else                ModifyMapPosY(-16);

        if (edit_pos_y == 4 && GetMapPosY() == 0) {
            edit_pos_y = 0;
        }
    }
    if (input & KEYFLAGS_DOWN) {
        if (edit_pos_y<80) edit_pos_y+=16;
        else               ModifyMapPosY(16);

        if (edit_pos_y == 80 && GetMapPosY() == GetMapPosY_MAX()) {
            edit_pos_y = 84;
        }
    }
    if (input & KEYFLAGS_LEFT) {
        if (edit_pos_x>15) edit_pos_x-=16;
        else               ModifyMapPosX(-16);
    }
    if (input & KEYFLAGS_RIGHT) {
        if (edit_pos_x<128) edit_pos_x+=16;
        else                ModifyMapPosX(16);
    }
    if (input & KEYFLAGS_KA) {
        SetMapContent(ScreenX2MapX(edit_pos_x),ScreenY2MapY(edit_pos_y),active_tile);
    }
    if (input & (KEYFLAGS_KB | KEYFLAGS_PLUS)) {
        active_tile++;
        if (active_tile>=nr_tiles) active_tile=0;
    }
    if (input & (KEYFLAGS_KC | KEYFLAGS_MINUS)) {
        if (!active_tile) active_tile = nr_tiles-1;
        else             active_tile--;
    }

    //WaitForAllKeysReleased();

    return COMMAND_REFRESH;
}


//=============================================================================
// simple callback for FileSelection dialog to filter just the tile files
// in the current folder
//=============================================================================
short TileFileCallback(unsigned char* src) {
    return (IS_TILEFILE(src)) ? 1 : 0;
}


//=============================================================================
// simple callback for FileSelection dialog to filter just the map files
// in the current folder
//=============================================================================
short MapFileCallback(unsigned char* src) {
    return (IS_MAPFILE(src)) ? 1 : 0;
}


//=============================================================================
// where all the fun starts ...
//=============================================================================
void _main(void) {
    short      i;
    TILEDATA*  tiledata;
    char       tmpname[9] = {0,0,0,0,0,0,0,0,0};
    MAPDATA*   mapdata;
    LCD_BUFFER screenbuffer;
    short      prg_exit = 0;
    SYM_ENTRY* selection;
    short      found;
    short      origfont = FontGetSys();

    //-------------------------------------------------------------------------
    // show mapfile selection dialog
    //-------------------------------------------------------------------------
    LCD_save(screenbuffer);
    switch(MainMenu()) {
        //*********************************************************************
        // USER WANTS TO EXIT
        //*********************************************************************
        case 0:
            LCD_restore(screenbuffer);
            return;
        //*********************************************************************
        // USER WANTS TO MODIFY A MAP
        //*********************************************************************
        case 1:
            //-------------------------------------------------------------------------
            // show mapfile selection dialog
            //-------------------------------------------------------------------------
            selection = FileSelection("Select Mapfile",MapFileCallback,&found);
            LCD_restore(screenbuffer);
            if (!selection) {
                if (!found) ST_showHelp("ERR: no mapfiles found");
                return;
            }
            strncpy(tmpname,selection->name,8);
            //-------------------------------------------------------------------------
            // load mapfile
            //-------------------------------------------------------------------------
            if (!(mapdata = LoadMapData(tmpname))) {
                ST_showHelp("ERR: cannot load map");
                return;
            }

            break;
        //*********************************************************************
        // USER WANTS TO CREATE A MAP
        //*********************************************************************
        default:
            i = SizeSelection();
            LCD_restore(screenbuffer);
            if (!i) return;

            if (!(mapdata = (MAPDATA*)malloc(i*i*2+14))) {
                ST_showHelp("ERR: out of mem");
                return;
            }
            memset(mapdata,0,i*i*2+14);
            mapdata->size_x = mapdata->size_y = i;

            //-------------------------------------------------------------------------
            // show tilefile selection dialog
            //-------------------------------------------------------------------------
            selection = FileSelection("Select Tilefile",TileFileCallback,&found);

            LCD_restore(screenbuffer);
            if (!selection) {
                if (!found) ST_showHelp("ERR: no tilefiles found");
                return;
            }
            strncpy(mapdata->tilefile,selection->name,8);
            break;
    }

    //-------------------------------------------------------------------------
    // load tilefile
    //-------------------------------------------------------------------------
    if (!(tiledata = LoadTileData(mapdata->tilefile))) {
        free(mapdata);
        ST_showHelp("ERR: cannot load tiledata");
        return;
    }

    //-------------------------------------------------------------------------
    // initializes GCKLibrary()
    //-------------------------------------------------------------------------
    if (!InitGCKLibrary()) {
        free(tiledata);
        free(mapdata);
        ST_showHelp("ERR: cannot initialized GCKLIB");
        return;
    }

    //-------------------------------------------------------------------------
    // turn off calling of default handlers
    //-------------------------------------------------------------------------
    SetDefaultINT1Exec(0);
    SetDefaultINT5Exec(0);

    //-------------------------------------------------------------------------
    // on the TI92p a border is drawn around the display
    //-------------------------------------------------------------------------
    if (!TI89) DrawBorder92();

    //-------------------------------------------------------------------------
    // setup map handling and enter main loop
    //-------------------------------------------------------------------------
    SetMap(mapdata,tiledata,0,0);

    // trick to overcome the "cannot edit behind tile list" problem
    // its very,very dirty to use the get function to modify the value
    GetMapPosX_MAX() += 16;

    edit_pos_x = 0;
    edit_pos_y = 0;
    active_tile = 0;
    nr_tiles    = tiledata->nr_tiles;

    while (!prg_exit) {
        switch(HandleKeys(GetKeyboardState())) {
            case COMMAND_EXIT:
                prg_exit = 1;
                break;
            case COMMAND_REFRESH:
            default:
                DrawMap();
                DrawRestOfScreen(tiledata->data);
                CopyBackbuffer2Screen();
                SleepForMilliseconds(50);
                break;
        }
    }

    //-------------------------------------------------------------------------
    // turn grayscales off, do some cleanup stuff and exit
    //-------------------------------------------------------------------------
    CleanupGCKLibrary();
    free(tiledata);
    free(mapdata);

    LCD_restore(screenbuffer);
    FontSetSys(origfont);

    ST_showHelp("powered by "GCKLIB_VERSION);
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log$
//