/******************************************************************************
*
* project name:  TICT Game Construction Kit (TICT-GCK)
* file name:     viewmap.c
* initial date:  27/06/2001
* author:        thomas.nussbaumer@gmx.net
* description:   simple map viewer
*
* use [+] and [-] to modify scroll speed
*
*
* $Id$
*
******************************************************************************/

#define NO_EXIT_SUPPORT
#define OPTIMIZE_ROM_CALLS

#include <tigcclib.h>
#include "../../lib/gcklib.h"

short _ti92plus;
short _ti89;

#define MINSTEP  1
#define MAXSTEP 48

short step = 1;

//=============================================================================
// handles input (returns 0 if user pressed [ESCAPE])
//=============================================================================
short HandleKeys(KBSTATE input) {
    if (input & KEYFLAGS_ESC) {
        WaitForAllKeysReleased();
        return 0;
    }

    if (input & KEYFLAGS_UP)    ModifyMapPosY(-step);
    if (input & KEYFLAGS_DOWN)  ModifyMapPosY(step);
    if (input & KEYFLAGS_LEFT)  ModifyMapPosX(-step);
    if (input & KEYFLAGS_RIGHT) ModifyMapPosX(step);
    if (input & KEYFLAGS_PLUS && step<MAXSTEP)  step++;
    if (input & KEYFLAGS_MINUS && step>MINSTEP) step--;

    return 1;
}


//=============================================================================
// simple callback for FileSelection dialog to filter just the map files
// in the current folder
//=============================================================================
short MapFileCallback(unsigned char* src) {
    return (IS_MAPFILE(src)) ? 1 : 0;
}


//=============================================================================
// where all the fun starts ...
//=============================================================================
void _main(void) {
    TILEDATA*  tiledata;
    char       mapname[9] = {0,0,0,0,0,0,0,0,0};
    MAPDATA*   mapdata;
    LCD_BUFFER screenbuffer;
    SYM_ENTRY* selection;
    short      found;

    //-------------------------------------------------------------------------
    // show mapfile selection dialog
    //-------------------------------------------------------------------------
    LCD_save(screenbuffer);
    selection = FileSelection("Select Mapfile",MapFileCallback,&found);
    LCD_restore(screenbuffer);
    if (selection == NULL) {
        if (found == 0) ST_showHelp("ERR: no mapfiles found");
        return;
    }

    strncpy(mapname,selection->name,8);

    //ESI        argptr;
    //short      argtype;

    //-------------------------------------------------------------------------
    // try to read commandline parameter (mapfile) or set default name
    //-------------------------------------------------------------------------
    //InitArgPtr(argptr);
    //argtype = GetArgType(argptr);
    //if (argtype != STR_TAG) mapname = "level";
    //else                    mapname = GetStrnArg(argptr);

    //-------------------------------------------------------------------------
    // load mapfile
    //-------------------------------------------------------------------------
    if (!(mapdata = LoadMapData(mapname))) {
        ST_showHelp("ERR: cannot load map");
        return;
    }

    //-------------------------------------------------------------------------
    // load tilefile
    //-------------------------------------------------------------------------
    if (!(tiledata = LoadTileData(mapdata->tilefile))) {
        free(mapdata);
        ST_showHelp("ERR: cannot load tiledata");
        return;
    }

    //-------------------------------------------------------------------------
    // initializes GCK library
    //-------------------------------------------------------------------------
    if (!InitGCKLibrary()) {
        free(tiledata);
        free(mapdata);
        ST_showHelp("ERR: cannot allocate buffer");
        return;
    }

    //-------------------------------------------------------------------------
    // turn off calling of default handlers
    //-------------------------------------------------------------------------
    SetDefaultINT1Exec(0);
    SetDefaultINT5Exec(0);

    //-------------------------------------------------------------------------
    // on the TI92p a border is drawn around the display
    //-------------------------------------------------------------------------
    if (!TI89) DrawBorder92();

    //-------------------------------------------------------------------------
    // setup map handling and enter main loop
    //-------------------------------------------------------------------------
    SetMap(mapdata,tiledata,0,0);

    while (1) {
        DrawMap();
        CopyBackbuffer2Screen();
        //printf_xy(0,0,"%03lu",GetActualTicks());
        if (!HandleKeys(GetKeyboardState())) break;
    }

    //-------------------------------------------------------------------------
    // do some cleanup stuff and exit
    //-------------------------------------------------------------------------
    CleanupGCKLibrary();
    free(tiledata);
    free(mapdata);

    LCD_restore(screenbuffer);
    ST_showHelp("powered by "GCKLIB_VERSION);
}

//#############################################################################
// Revision History
//#############################################################################
//
// $Log$
//
